//////////////////////////////////////////////////////////////////////////////////////
////////////  Nick Marchuk  //////////////////////////////////////////////////////////
////////////  NU32v2 Stripchart Plotting Application   ///////////////////////////////
////////////  February 2011 //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

// serial library
import processing.serial.*; 
Serial myPort; 

// buttons and labels
TextButtons[] serialButtons;
TextButtons printButton;
PFont bigfont, smallfont, divfont;
String serialPortLabel, sentLabel, gotLabel;
int number_serial_buttons;

// boolean controls for program control
boolean SerialOK;

// plot buffers
int[] buffA = new int[1000];
int[] buffB = new int[1000];
int[] buffC = new int[1000];
int[] buffD = new int[1000];
int[] buffE = new int[1000];
int[] buffF = new int[1000];
int rollingIndex;

//////////////////////////////////////////////////////////////////////////////////////
////////////  setup  /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

void setup()
{
  size(1030, 602);
  background(230, 230, 230);
  frameRate(30);

  bigfont = loadFont("Gisha-Bold-24.vlw"); 
  smallfont = loadFont("Gisha-Bold-16.vlw");
  divfont = loadFont("Gisha-Bold-8.vlw");

  SerialOK = false;

  serialPortLabel = "No serial port selected, select from 'Serial Ports'";
  sentLabel = "Sent: ";
  gotLabel = "Got: ";

  // serial
  println(Serial.list());
  number_serial_buttons = Serial.list().length;
  serialButtons = new TextButtons[number_serial_buttons];
  int index_serial = 0;
  for (index_serial = 0; index_serial < number_serial_buttons; index_serial++) {
    serialButtons[index_serial] = new TextButtons(50,50+index_serial*40,165,35,bigfont,Serial.list()[index_serial]); // x,y,w,h,font,text
  }

  printButton = new TextButtons(850,545,155,35,bigfont,"Print Screen");

  for (int i = 0; i < 1000; i++) {
    buffA[i] = 0;
    buffB[i] = 10;
    buffC[i] = 20;
    buffD[i] = 30;
    buffE[i] = 40;
    buffF[i] = 50;
  }

  rollingIndex = 0;
}

//////////////////////////////////////////////////////////////////////////////////////
////////////  draw  //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

void draw()
{
  int i = 0;
  int index = 0;
  int plot = 0;
  background(230, 230, 230);

  // status background and text
  fill(255);
  rect(10,532,1000,65);
  fill(0);
  textFont(smallfont);
  text("Status",15,547);

  // status label
  if (SerialOK) {
    fill(15,190,30);
    text(serialPortLabel,15,562); // serial port
    fill(0);
    text(sentLabel,15,580); // what has been sent
    text(gotLabel,15,595); // what has been sent
    text("Plotting",450,562);
    fill(255,0,0);  // red
    text("a",520,562);
    fill(0,255,0);  // green
    text("b",530,562);
    fill(0,0,255);  // blue
    text("c",540,562);
    fill(150,15,150);  // purple
    text("d",550,562);
    fill(252,156,0);  // orange
    text("e",560,562);
    fill(0,252,246);  // light blue
    text("f",570,562);
  }
  else {
    fill(165,5,5);
    text(serialPortLabel,15,562); // serial port
  }

  // serial buttons
  if (!SerialOK) {
    fill(255);
    rect(45,15,175,35+number_serial_buttons*40); 
    fill(0);
    textFont(smallfont);
    text("Serial Ports",55,40);
    for (index = 0; index < number_serial_buttons; index++) {
      serialButtons[index].update();
      serialButtons[index].display();
    }
    textFont(smallfont);
    fill(0);
    text("Welcome to the NU32v2 Stripchart tool",255,40);

    text("This tool plots six traces, with 1000 values in the x axis and 512 values in the y axis.",255,90);
    text("",255,115);
    //115
    text("Send a string of six integers, separated by spaces, like this:",255,140);
    text("       a b c d e f\\r\\n",255,165);
    text("which will plot a point in each trace on the right and shift the other points to the left.",255,190);
    // 215
    text("Trace a will plot in red, b in green, c in blue, d in purple, e in orange, and f in light blue.",255,240);
    text("The integer value must be between 0 and 511.",255,265);
    // 290
    text("Typing a key will send the pressed character over serial to the microcontroller.",255,315);
    //text("A message received from the microcontroller will be printed in the \"Got:\" section.",255,340);
    //text("Messages received from the microcontroller cannot begin with a, b, c, or d.",255,365); 
    // 390
    text("The 'Print Screen' button will save a .png image of the screen.",255,415);
    // 440
    text("Select your COM port on the left to begin.",255,465);
  }
  else {
    printButton.update();
    printButton.display();
    ///////////////////////////////////  Draw the plots /////////////////////////////////////////////////////////////
    textFont(divfont);
    fill(255);
    rect(10,10,1000,512);
    for (i = 10; i < 1000; i=i+100) {
      line(i,10,i,20);
      line(i,512,i,522);
    }
    for (i = 522; i > 10; i=i-100) {
      line(10,i,20,i);
      line(1000,i,1010,i);
    }

    fill(0);
    text("0",1012,525);
    text("100",1012,425);
    text("200",1012,325);
    text("300",1012,225);
    text("400",1012,125);
    text("500",1012,25);

    text("0",10,8);
    text("300",302,8);
    text("500",502,8);
    text("700",702,8);
    text("1000",998,8);

    noStroke();

    for (i = rollingIndex; i < 1000; i++) {
      fill(255,0,0);  // red
      ellipse(plot+10,522-buffA[i],2,2);

      fill(0,255,0);  // green
      ellipse(plot+10,522-buffB[i],2,2);

      fill(0,0,255);  // blue
      ellipse(plot+10,522-buffC[i],2,2);

      fill(150,15,150);  // purple
      ellipse(plot+10,522-buffD[i],2,2);

      fill(252,156,0);  // orange
      ellipse(plot+10,522-buffE[i],2,2);

      fill(0,252,246);  // light blue
      ellipse(plot+10,522-buffF[i],2,2);

      plot++;
    }

    for (i = 0; i < rollingIndex; i++) {
      fill(255,0,0);  // red
      ellipse(plot+10,522-buffA[i],2,2);

      fill(0,255,0);  // green
      ellipse(plot+10,522-buffB[i],2,2);

      fill(0,0,255);  // blue
      ellipse(plot+10,522-buffC[i],2,2);

      fill(150,15,150);  // purple
      ellipse(plot+10,522-buffD[i],2,2);

      fill(252,156,0);  // orange
      ellipse(plot+10,522-buffE[i],2,2);

      fill(0,252,246);  // light blue
      ellipse(plot+10,522-buffF[i],2,2);

      plot++;
    }
  }
  stroke(0);
}

//////////////////////////////////////////////////////////////////////////////////////
////////////  mouseClicked  //////////////////////////////////////////////////////////
////////////  If a button is selected,  //////////////////////////////////////////////
////////////  perform a function releated to the buttons name  ///////////////////////
//////////////////////////////////////////////////////////////////////////////////////

void mousePressed() {
  try {
    String buttonText = null;
    int index = 0;
    boolean serialButton = false;

    // check all of the buttons
    for (index = 0; index < number_serial_buttons; index++) {
      if (serialButtons[index].over()) {
        serialButton = true;
        buttonText = serialButtons[index].thetext;
      }
    }

    if (printButton.over()) {
      buttonText = printButton.thetext;
    }

    if (buttonText == null) {
      // nothing, no button was clicked
    }
    else {
      /////////////////////////////// Serial Button //////////////////////////////////////////////////////
      if (serialButton) {
        myPort = new Serial(this, buttonText, 115200);
        myPort.bufferUntil('\n'); 
        serialPortLabel = "'" + buttonText + "' selected";
        SerialOK = true;
      }
      ///////////////////////////// Other button /////////////////////////////////////////////////////////
      if (buttonText.equals("Print Screen")) {
        save("printscreen.png");
      }
    }
  }
  catch (NullPointerException e) {
    println("NullPointerException in MousePressed");
    exit();
  }
}

//////////////////////////////////////////////////////////////////////////////////////
////////////  key pressed ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
void keyReleased() { 

  if (SerialOK) {
    myPort.write(key);
    sentLabel = sentLabel + key;
  }
  else { // not in serial mode yet
    if (key == 'e') {
      exit();
    }
    if (key == '1') {
      myPort = new Serial(this, Serial.list()[0], 115200);
      myPort.bufferUntil('\n'); 
      serialPortLabel = "'" + Serial.list()[0] + "' selected";
      SerialOK = true;
    }
  }
}

//////////////////////////////////////////////////////////////////////////////////////
////////////  serial event   /////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

void serialEvent(Serial thisPort) { 
  try {
    if (SerialOK) {
      // read the serial buffer as a string until a newline appears
      String myString = thisPort.readStringUntil('\n');

      // if you got any bytes other than the newline
      if (myString != null) {

        myString = trim(myString); // ditch the newline
        if (myString.substring(0,1).equals("m")) {
          gotLabel = "Got: " + myString.substring(2,myString.length());
          //println(myString.substring(2,myString.length()));
        }
        else {
          int sensors[] = int(split(myString, ' '));

          if (sensors.length == 0) {
            buffA[rollingIndex] = 0;
            buffB[rollingIndex] = 0;
            buffC[rollingIndex] = 0;
            buffD[rollingIndex] = 0;
            buffE[rollingIndex] = 0;
            buffF[rollingIndex] = 0;
          }

          if (sensors.length == 1) {
            buffA[rollingIndex] = sensors[0];
            buffB[rollingIndex] = 0;
            buffC[rollingIndex] = 0;
            buffD[rollingIndex] = 0;
            buffE[rollingIndex] = 0;
            buffF[rollingIndex] = 0;
          }

          if (sensors.length == 2) {
            buffA[rollingIndex] = sensors[0];
            buffB[rollingIndex] = sensors[1];
            buffC[rollingIndex] = 0;
            buffD[rollingIndex] = 0;
            buffE[rollingIndex] = 0;
            buffF[rollingIndex] = 0;
          }

          if (sensors.length == 3) {
            buffA[rollingIndex] = sensors[0];
            buffB[rollingIndex] = sensors[1];
            buffC[rollingIndex] = sensors[2];
            buffD[rollingIndex] = 0;
            buffE[rollingIndex] = 0;
            buffF[rollingIndex] = 0;
          }

          if (sensors.length == 4) {
            buffA[rollingIndex] = sensors[0];
            buffB[rollingIndex] = sensors[1];
            buffC[rollingIndex] = sensors[2];
            buffD[rollingIndex] = sensors[3];
            buffE[rollingIndex] = 0;
            buffF[rollingIndex] = 0;
          }

          if (sensors.length == 5) {
            buffA[rollingIndex] = sensors[0];
            buffB[rollingIndex] = sensors[1];
            buffC[rollingIndex] = sensors[2];
            buffD[rollingIndex] = sensors[3];
            buffE[rollingIndex] = sensors[4];
            buffF[rollingIndex] = 0;
          }
          if (sensors.length == 6) {
            buffA[rollingIndex] = sensors[0];
            buffB[rollingIndex] = sensors[1];
            buffC[rollingIndex] = sensors[2];
            buffD[rollingIndex] = sensors[3];
            buffE[rollingIndex] = sensors[4];
            buffF[rollingIndex] = sensors[5];
          }

          rollingIndex++;
          if (rollingIndex == 1000) {
            rollingIndex = 0;
          }
        }
      }
    }
  }
  catch (NullPointerException e) {
    println("nullpointer in serial");
  }
} // end serialEvent

//////////////////////////////////////////////////////////////////////////////////////
////////////  exit   /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////


public void stop() {
  //println("inside stop");
  try {
    if (SerialOK) {
      // Clear the buffer, or available() will still be > 0
      myPort.clear();
      // Close the port
      myPort.stop();
    }
    super.stop();
    //println("exiting...");
  }
  catch (NullPointerException e) {
    println("NullPointerException in stop");
  }
} 

//////////////////////////////////////////////////////////////////////////////////////
////////////  button class  //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

class Button
{
  int x, y;
  int w, h;
  color basecolor, highlightcolor, clickcolor;
  PFont thefont;
  boolean over = false;
  boolean pressed = false;   

  void pressed() {
    if(over && mousePressed) {
      pressed = true;
    } 
    else {
      pressed = false;
    }
  }

  boolean overRect(int x, int y, int width, int height) {
    if (mouseX >= x && mouseX <= x+width && 
      mouseY >= y && mouseY <= y+height) {
      return true;
    } 
    else {
      return false;
    }
  }
}

class TextButtons extends Button 
{
  String thetext;
  color bgcolor;

  TextButtons(int ix, int iy, int iw, int ih, PFont ithefont, String itext) 
  {
    x = ix;
    y = iy;
    w = iw;
    h = ih;
    thefont = ithefont;
    thetext = itext;
    basecolor = color(52,173,232);
    highlightcolor = color(0,102,153);
    clickcolor = color(152,200,224);//(10,240,100);
  }

  void update() 
  {
    over();
    pressed();
    if(pressed) {
      bgcolor = clickcolor;
    } 
    else if (over) {
      bgcolor = highlightcolor;
    } 
    else {
      bgcolor = basecolor;
    }
  }

  boolean over() 
  {
    if( overRect(x, y, w, h) ) {
      over = true;
      return true;
    } 
    else {
      over = false;
      return false;
    }
  }

  void display() 
  {
    textFont(thefont);
    fill(bgcolor);
    rect(x,y,w,h);
    fill(255,255,255);
    text(thetext,x+5,y+h-10);
  }
}

